@@ -96,7 +96,7 @@ def Coerce(target_type, msg=None):
                 return v
             return target_type(v)
         except ValueError:
-            raise Invalid(msg or ('expected %s' % target_type.__name__))
+            raise Invalid(msg or ('Failed to coerce %s of type \'%s\' to \'%s\'' % (repr(v), type(v).__name__, target_type.__name__)))
     return f
 
 
@@ -108,7 +108,7 @@ def IsTrue(msg=None):
     def f(v):
         if v:
             return v
-        raise Invalid(msg or 'value was not true')
+        raise Invalid(msg or '%s is not True' % (repr(v)))
     return f
 
 
@@ -118,7 +118,7 @@ def IsFalse(msg=None):
     def f(v):
         if not v:
             return v
-        raise Invalid(msg or 'value was not false')
+        raise Invalid(msg or '%s is not False' % (repr(v)))
     return f
 
 
@@ -136,10 +136,10 @@ def Boolean(msg=None):
                     return True
                 if v in ('0', 'false', 'no', 'off', 'disable'):
                     return False
-                raise Invalid(msg or 'expected boolean')
+                raise Invalid(msg or ('Failed to parse %s of type \'%s\' as Boolean' % (repr(v), type(v).__name__)))
             return bool(v)
         except ValueError:
-            raise Invalid(msg or 'expected boolean')
+            raise Invalid(msg or ('Failed to parse %s of type \'%s\' as Boolean' % (repr(v), type(v).__name__)))
     return f
 
 
@@ -161,7 +161,7 @@ def Any(*validators, **kwargs):
                     raise
                 pass  # pragma: no coverage (still and again a coverage bug)
         else:
-            raise Invalid(msg or 'no valid value found')
+            raise Invalid(msg or 'no validator matched for %s of type \'%s\'' % (repr(v), type(v).__name__))
     return f
 
 
@@ -195,7 +195,7 @@ def Match(pattern, msg=None):
 
     def f(v):
         if not pattern.match(v):
-            raise Invalid(msg or 'does not match regular expression')
+            raise Invalid(msg or '%s does not match regular expression %s' % (repr(v), type(v).__name__))
         return v
     return f
 
@@ -218,7 +218,7 @@ def Url(msg=None):
             urlparse.urlparse(v)
             return v
         except:
-            raise Invalid(msg or 'expected a URL')
+            raise Invalid(msg or '%s does not look like a valid URL' % (repr(v)))
     return f
 
 
@@ -228,7 +228,7 @@ def IsFile(msg=None):
         if os.path.isfile(v):
             return v
         else:
-            raise Invalid(msg or 'not a file')
+            raise Invalid(msg or '%s does not point to a file' % (repr(v)))
     return f
 
 
@@ -238,7 +238,7 @@ def IsDir(msg=None):
         if os.path.isdir(v):
             return v
         else:
-            raise Invalid(msg or 'not a directory')
+            raise Invalid(msg or '%s does not point to a directory' % (repr(v)))
     return f
 
 
@@ -248,7 +248,7 @@ def PathExists(msg=None):
         if os.path.exists(v):
             return v
         else:
-            raise Invalid(msg or 'path does not exist')
+            raise Invalid(msg or '%s does not exist on filesystem' % (repr(v)))
     return f
 
 
@@ -261,9 +261,9 @@ def InRange(min=None, max=None, msg=None):
     """
     def f(v):
         if min is not None and v < min:
-            raise Invalid(msg or 'value must be at least %s' % min)
+            raise Invalid(msg or 'value must be at least %s. Got %s' % (min, repr(v)))
         if max is not None and v > max:
-            raise Invalid(msg or 'value must be at most %s' % max)
+            raise Invalid(msg or 'value must be at most %s. Got %s' % (max, repr(v)))
         return v
     return f
 
@@ -286,9 +286,9 @@ def Length(min=None, max=None, msg=None):
     """The length of a value must be in a certain range."""
     def f(v):
         if min is not None and len(v) < min:
-            raise Invalid(msg or 'length of value must be at least %s' % min)
+            raise Invalid(msg or 'length of value must be at least %s. Got %s of length %s' % (min, repr(v), len(v)))
         if max is not None and len(v) > max:
-            raise Invalid(msg or 'length of value must be at most %s' % max)
+            raise Invalid(msg or 'length of value must be at most %s. Got %s of length %s' % (max, repr(v), len(v)))
         return v
     return f
 
